@@ -1,4 +1,4 @@
-"""Modules for analzing VASP outputs."""
+"""Modulet for analzing VASP outputs."""
 
 from scipy.constants import physical_constants
 from scipy.constants import speed_of_light
@@ -11,11 +11,14 @@ from jarvis.core.kpoints import Kpoints3D as Kpoints
 from jarvis.io.vasp.inputs import Poscar
 from matplotlib import pyplot as plt
 from jarvis.core.utils import rec_dict
+from jarvis.core.utils import recast_array_on_uniq_array_elements
+import scipy.signal as ss
 
 RYTOEV = 13.605826
 AUTOA = 0.529177249
 TPI = 2 * np.pi
 HSQDTM = RYTOEV * AUTOA * AUTOA
+plt.switch_backend("agg")
 
 
 class Chgcar(object):
@@ -162,6 +165,80 @@ class Chgcar(object):
         return tmp
 
 
+class Locpot(Chgcar):
+    """Read LOCPOT files."""
+
+    def vac_potential(
+        self, direction="X", Ef=0, filename="Avg.png", plot=True
+    ):
+        """Calculate vacuum potential used in work-function calculation."""
+        atoms = self.atoms
+        cell = atoms.lattice_mat
+        chg = (self.chg[-1].T) * atoms.volume
+        latticelength = np.dot(cell, cell.T).diagonal()
+        latticelength = latticelength ** 0.5
+        ngridpts = np.array(chg.shape)
+        # totgridpts = ngridpts.prod()
+
+        if direction == "X":
+            idir = 0
+            a = 1
+            b = 2
+        elif direction == "Y":
+            a = 0
+            idir = 1
+            b = 2
+        else:
+            a = 0
+            b = 1
+            idir = 2
+        a = (idir + 1) % 3
+        b = (idir + 2) % 3
+        average = np.zeros(ngridpts[idir], np.float)
+        for ipt in range(ngridpts[idir]):
+            if direction == "X":
+                average[ipt] = chg[ipt, :, :].sum()
+            elif direction == "Y":
+                average[ipt] = chg[:, ipt, :].sum()
+            else:
+                average[ipt] = chg[:, :, ipt].sum()
+        average /= ngridpts[a] * ngridpts[b]
+        xdiff = latticelength[idir] / float(ngridpts[idir] - 1)
+        xs = []
+        ys = []
+        for i in range(ngridpts[idir]):
+            x = i * xdiff
+            xs.append(x)
+            ys.append(average[i])
+
+        avg_max = max(average)
+
+        dif = float(avg_max) - float(Ef)
+        if plt:
+            plt.xlabel("z (Angstrom)")
+            plt.plot(xs, ys, "-", linewidth=2, markersize=10)
+            horiz_line_data = np.array([avg_max for i in range(len(xs))])
+            plt.plot(xs, horiz_line_data, "-")
+            horiz_line_data = np.array([Ef for i in range(len(xs))])
+            plt.plot(xs, horiz_line_data, "-")
+            plt.ylabel("Potential (eV)")
+            ax = plt.gca()
+            ax.get_yaxis().get_major_formatter().set_useOffset(False)
+            plt.title(
+                str("Energy difference ")
+                + str(round(float(dif), 3))
+                + str(" eV"),
+                fontsize=26,
+            )
+            plt.tight_layout()
+
+            plt.savefig(filename)
+            plt.close()
+
+        print("Ef,max,wf=", Ef, avg_max, dif)
+        return avg_max, dif
+
+
 class Oszicar(object):
     """Construct Oszicar object."""
 
@@ -276,9 +353,12 @@ class Outcar(object):
             pass
         return cnvg
 
-    @property
-    def efg_tensor_diag(self):
-        """Get electric field gradient tensor."""
+    #  @property
+    def efg_tensor_diag(self, std_conv=True, prec=3):
+        """Get diagonalized electric field gradient tensor."""
+        # std_conv: |Vzz|>=|Vyy|>=|Vxx|, eta=(Vxx-Vyy)/Vzz
+        # Note: VASP uses: |Vzz|>=|Vxx|>=|Vyy|, eta=(Vyy-Vxx)/Vzz
+        # quadrupolar parameter, Cq=e*Q*V_zz/h
         nions = self.nions
         for ii, i in enumerate(self.data):
             if "Electric field gradients after diagonalization" in i:
@@ -286,7 +366,44 @@ class Outcar(object):
         arr = self.data[tmp + 5 : tmp + 5 + nions]
         efg_arr = []
         for i in arr:
-            tmp = [i.split()[1], i.split()[2], i.split()[3], i.split()[4]]
+            if std_conv:
+                Vzz = round(float(i.split()[3]), prec)
+                Vyy = round(float(i.split()[1]), prec)
+                Vxx = round(float(i.split()[2]), prec)
+                if Vzz == 0.0:
+                    eta = 0.0
+                else:
+                    eta = (Vxx - Vyy) / Vzz
+                tmp = [Vxx, Vyy, Vzz, eta]
+            else:
+                Vzz = round(float(i.split()[3]), prec)
+                Vyy = round(float(i.split()[2]), prec)
+                Vxx = round(float(i.split()[1]), prec)
+                if Vzz == 0.0:
+                    eta = 0.0
+                else:
+                    eta = (Vyy - Vxx) / Vzz
+                tmp = [Vxx, Vyy, Vzz, eta]
+            efg_arr.append(tmp)
+        efg_arr = np.array(efg_arr, dtype="float")
+        return efg_arr
+
+    @property
+    def efg_raw_tensor(self):
+        """Get raw electric field gradient tensor."""
+        nions = self.nions
+        for ii, i in enumerate(self.data):
+            if "Electric field gradients (V/A^2)" in i:
+                tmp = ii
+        arr = self.data[tmp + 4 : tmp + 4 + nions]
+        efg_arr = []
+        for i in arr:
+            line = i.split()
+            tmp = [
+                [line[1], line[4], line[5]],
+                [line[4], line[2], line[6]],
+                [line[5], line[6], line[3]],
+            ]
             efg_arr.append(tmp)
         efg_arr = np.array(efg_arr, dtype="float")
         return efg_arr
@@ -359,6 +476,7 @@ class Outcar(object):
         v = open(self.filename, "r")
         lines = v.read().splitlines()
         c = np.empty((6, 6), dtype=float)
+        # TODO: Use regex to simplify
         for i, line in enumerate(lines):
             if "TOTAL ELASTIC MODULI (kBar)" in line:
                 c11 = lines[i + 3].split()[1]
@@ -986,13 +1104,12 @@ class Vasprun(object):
         )
         return energies, absorption
 
-    @property
-    def dfpt_data(self, fc_mass=True):
-        """Get DFPT IBRION=8 related data."""
+    def phonon_data(self, fc_mass=True):
+        """Get phonon data."""
         info = {}
         hessian = []
-        data = self._data
-        for i in (data["modeling"]["calculation"]["dynmat"]["varray"])[0]["v"]:
+        # data = self._data
+        for i in (self.ionic_steps[-1]["dynmat"]["varray"])[0]["v"]:
             hessian.append(i.split())
         hessian = np.array(hessian, dtype="double")
         struct = self.all_structures[-1]
@@ -1008,6 +1125,32 @@ class Vasprun(object):
             for i in range(natoms):
                 for j in range(natoms):
                     force_constants[i, j] *= -np.sqrt(masses[i] * masses[j])
+        phonon_eigenvals = np.array(
+            self.ionic_steps[-1]["dynmat"]["v"]["#text"].split(),
+            dtype="double",
+        )
+        eigvecs = np.array(
+            [
+                i.split()
+                for i in (self.ionic_steps[-1]["dynmat"]["varray"][1]["v"])
+            ],
+            dtype="float",
+        )
+        phonon_eigenvectors = []
+        for ev in eigvecs:
+            phonon_eigenvectors.append(np.array(ev).reshape(natoms, 3))
+        info["phonon_eigenvectors"] = phonon_eigenvectors
+        info["phonon_eigenvalues"] = phonon_eigenvals
+        info["masses"] = masses
+        info["force_constants"] = force_constants
+        return info
+
+    @property
+    def dfpt_data(self, fc_mass=True):
+        """Get DFPT IBRION=8 LEPSILON lated data."""
+        info = self.phonon_data(fc_mass=fc_mass)
+        data = self._data
+        natoms = self.num_atoms
         born_charges = []
         for n in range(natoms):
             born_charges.append(
@@ -1019,23 +1162,6 @@ class Vasprun(object):
                 ]
             )
         born_charges = np.array(born_charges, dtype="double")
-        phonon_eigenvals = np.array(
-            data["modeling"]["calculation"]["dynmat"]["v"]["#text"].split(),
-            dtype="double",
-        )
-        eigvecs = np.array(
-            [
-                i.split()
-                for i in (
-                    data["modeling"]["calculation"]["dynmat"]["varray"][1]["v"]
-                )
-            ],
-            dtype="float",
-        )
-        phonon_eigenvectors = []
-        for ev in eigvecs:
-            phonon_eigenvectors.append(np.array(ev).reshape(natoms, 3))
-        phonon_eigenvectors = np.array(phonon_eigenvectors, dtype="float")
         epsilon = {}
         for i in data["modeling"]["calculation"]["varray"]:
             if "epsilon" in i["@name"]:
@@ -1043,10 +1169,7 @@ class Vasprun(object):
                     [j.split() for j in i["v"]], dtype="float"
                 )
         info["born_charges"] = born_charges
-        info["phonon_eigenvectors"] = phonon_eigenvectors
         info["epsilon"] = epsilon
-        info["phonon_eigenvalues"] = phonon_eigenvals
-        info["masses"] = masses
         return info
 
     @property
@@ -1091,10 +1214,11 @@ class Vasprun(object):
                 float(self.all_input_parameters["NELECT"])
                 / float(spin_channels)
             )
-            print("levels", levels)
             gap = min(self.eigenvalues[0][:, :, 0][:, levels]) - max(
                 self.eigenvalues[0][:, :, 0][:, levels - 1]
             )
+            vbm = max(self.eigenvalues[0][:, :, 0][:, levels - 1])
+            cbm = min(self.eigenvalues[0][:, :, 0][:, levels])
 
         if self.is_spin_polarized:
             tmp = np.concatenate(
@@ -1103,8 +1227,10 @@ class Vasprun(object):
             )
             cat = np.sort(tmp, axis=1)
             nelect = int(float(self.all_input_parameters["NELECT"]))
+            cbm = max(cat[:, nelect - 1])
+            vbm = min(cat[:, nelect])
             gap = min(cat[:, nelect]) - max(cat[:, nelect - 1])
-        return gap
+        return gap, cbm, vbm
 
     @property
     def fermi_velocities(self):
@@ -1337,15 +1463,16 @@ class Vasprun(object):
         E_high=4,
         spin=0,
         zero_efermi=True,
-        kpoints_file_path=".",
+        kpoints_file_path="KPOINTS",
+        plot=False,
     ):
         """Get electronic bandstructure plot."""
         try:
+            kp_labels = []
+            kp_labels_points = []
             f = open(kpoints_file_path, "r")
             lines = f.read().splitlines()
             f.close()
-            kp_labels = []
-            kp_labels_points = []
             for ii, i in enumerate(lines):
                 if ii > 2:
                     tmp = i.split()
@@ -1363,28 +1490,57 @@ class Vasprun(object):
             pass
 
         tmp = 0.0
+        info = {}
+        info["efermi"] = float(self.efermi)
         if zero_efermi:
             tmp = float(self.efermi)
-        plt.clf()
-        for i, ii in enumerate(self.eigenvalues[spin][:, :, 0].T - tmp):
-            plt.plot(ii, color="r")
 
+        spin_up_bands_x = []
+        spin_up_bands_y = []
+        spin_down_bands_x = []
+        spin_down_bands_y = []
+        for i, ii in enumerate(self.eigenvalues[spin][:, :, 0].T - tmp):
+            # plt.plot(ii, color="r")
+            spin_up_bands_x.append([np.arange(0, len(ii))])
+            spin_up_bands_y.append([ii])
         if self.is_spin_polarized:
             for i, ii in enumerate(self.eigenvalues[1][:, :, 0].T - tmp):
-                plt.plot(ii, color="b")
+                # plt.plot(ii, color="b")
+                spin_down_bands_x.append([np.arange(0, len(ii))])
+                spin_down_bands_y.append([ii])
 
-        plt.ylim([E_low, E_high])
-        plt.xticks(kp_labels_points, kp_labels)
-        plt.xlim([0, len(self.kpoints._kpoints)])
-        plt.xlabel(r"$\mathrm{Wave\ Vector}$")
-        ylabel = (
-            r"$\mathrm{E\ -\ E_f\ (eV)}$"
-            if zero_efermi
-            else r"$\mathrm{Energy\ (eV)}$"
-        )
-        plt.ylabel(ylabel)
+        info["spin_up_bands_x"] = spin_up_bands_x
+        info["spin_up_bands_y"] = spin_up_bands_y
+        info["spin_down_bands_x"] = spin_down_bands_x
+        info["spin_down_bands_y"] = spin_down_bands_y
 
-        return plt
+        info["kp_labels_points"] = list(kp_labels_points)
+        info["kp_labels"] = list(kp_labels)
+        if plot:
+            for i, j in zip(info["spin_up_bands_x"], info["spin_up_bands_y"]):
+                plt.plot(
+                    np.array(i).flatten(), np.array(j).flatten(), color="b"
+                )
+
+            if self.is_spin_polarized:
+                for i, j in zip(
+                    info["spin_down_bands_x"], info["spin_down_bands_y"]
+                ):
+                    plt.plot(
+                        np.array(i).flatten(), np.array(j).flatten(), color="r"
+                    )
+
+            plt.ylim([E_low, E_high])
+            plt.xticks(kp_labels_points, kp_labels)
+            plt.xlim([0, len(self.kpoints._kpoints)])
+            plt.xlabel(r"$\mathrm{Wave\ Vector}$")
+            ylabel = (
+                r"$\mathrm{E\ -\ E_f\ (eV)}$"
+                if zero_efermi
+                else r"$\mathrm{Energy\ (eV)}$"
+            )
+            plt.ylabel(ylabel)
+        return info
 
     @property
     def total_dos(self):
@@ -1492,14 +1648,185 @@ class Vasprun(object):
                             )
         return info
 
+    def get_spdf_dos(self, plot=False):
+        """Get spdf resolved partial density of states."""
+        info = {}
+        s = ["s"]
+        p = ["px", "py", "pz"]
+        d = ["dxy", "dyz", "dxz", "dz2", "dx2"]
+        f = ["f-3", "f-2", "f-1", "f0", "f1", "f2", "f3"]
+        # spin = 0
+        spin_pol = self.is_spin_polarized
+        num_atoms = self.all_structures[-1].num_atoms
+        pdos = self.partial_dos_spdf  # spin,atom,spdf
+        energy = pdos[0][0]["energy"] - self.efermi
+        spin_up_s = np.zeros(len(energy))
+        spin_up_p = np.zeros(len(energy))
+        spin_up_d = np.zeros(len(energy))
+        has_f_elements = False
+        if "f1" in pdos[0][0].keys():
+            has_f_elements = True
+            spin_up_f = np.zeros(len(energy))
+        for i in range(num_atoms):
+            for j in s:
+                spin_up_s += pdos[0][i][j]
+            for j in p:
+                spin_up_p += pdos[0][i][j]
+            for j in d:
+                spin_up_d += pdos[0][i][j]
+            if has_f_elements:
+                for j in f:
+                    spin_up_f += pdos[0][i][j]
+        info["spin_up_s"] = spin_up_s
+        info["spin_up_p"] = spin_up_p
+        info["spin_up_d"] = spin_up_d
+        info["energy"] = energy
+        if has_f_elements:
+            info["spin_up_f"] = spin_up_f
+        if spin_pol:
+            # spin = 1
+            spin_down_s = np.zeros(len(energy))
+            spin_down_p = np.zeros(len(energy))
+            spin_down_d = np.zeros(len(energy))
+            if has_f_elements:
+                spin_down_f = np.zeros(len(energy))
+            for i in range(num_atoms):
+                for j in s:
+                    spin_down_s += pdos[1][i][j]
+                for j in p:
+                    spin_down_p += pdos[1][i][j]
+                for j in d:
+                    spin_down_d += pdos[1][i][j]
+                if has_f_elements:
+                    for j in f:
+                        spin_down_f += pdos[1][i][j]
 
-"""
-if __name__ == "__main__":
-    vrun = Vasprun(
-        "vasprun.xml"
-    )
-    _, _, tdos = vrun.total_dos
-    pdos = vrun.partial_dos_spdf
-    pdos = vrun.projected_atoms_spins_kpoints_bands
-    pdos = vrun.projected_spins_kpoints_bands
-"""
+            info["spin_down_s"] = -1 * spin_down_s
+            info["spin_down_p"] = -1 * spin_down_p
+            info["spin_down_d"] = -1 * spin_down_d
+            info["has_f_elements"] = str(has_f_elements)
+            if has_f_elements:
+                info["spin_down_f"] = -1 * spin_down_f
+            if plot:
+                plt.plot(
+                    info["energy"], info["spin_up_s"], color="red", label="s"
+                )
+
+                plt.plot(
+                    info["energy"], info["spin_up_p"], color="green", label="p"
+                )
+
+                plt.plot(
+                    info["energy"], info["spin_up_d"], color="blue", label="d"
+                )
+                if has_f_elements:
+                    plt.plot(
+                        info["energy"],
+                        info["spin_up_f"],
+                        color="black",
+                        label="f",
+                    )
+                if spin_pol:
+                    plt.plot(info["energy"], info["spin_down_s"], color="red")
+                    plt.plot(
+                        info["energy"], info["spin_down_p"], color="green"
+                    )
+                    plt.plot(info["energy"], info["spin_down_d"], color="blue")
+                if has_f_elements:
+                    plt.plot(
+                        info["energy"], info["spin_down_f"], color="black"
+                    )
+                plt.xlim([-5, 10])
+                plt.legend()
+        return info
+
+    def get_atom_resolved_dos(self, plot=False):
+        """Get atom resolved density of states."""
+        # spin_pol = self.is_spin_polarized
+        atoms = self.all_structures[-1]
+        # num_atoms = atoms.num_atoms
+        elements = atoms.elements
+        unique_elements = atoms.uniq_species
+        pdos = self.partial_dos_spdf  # spin,atom,spdf
+        energy = pdos[0][0]["energy"] - self.efermi
+        element_dict = recast_array_on_uniq_array_elements(
+            unique_elements, elements
+        )
+        valid_keys = []
+        info = {}
+        info["spin_up_info"] = {}
+        info["spin_down_info"] = {}
+        info["energy"] = energy
+        for i in pdos[0][0].keys():
+            if "energ" not in i:
+                valid_keys.append(i)
+        # print (valid_keys)
+        spin_up_info = {}
+        for i, j in element_dict.items():
+            spin_up_info[i] = np.zeros(len(energy))
+
+        for i, j in element_dict.items():
+            for atom in j:
+                for k in valid_keys:
+                    spin_up_info[i] += pdos[0][atom][k]
+        info["spin_up_info"] = spin_up_info
+        if self.is_spin_polarized:
+            spin_down_info = {}
+            for i, j in element_dict.items():
+                spin_down_info[i] = np.zeros(len(energy))
+
+            for i, j in element_dict.items():
+                for atom in j:
+                    for k in valid_keys:
+                        spin_down_info[i] += -1 * pdos[0][atom][k]
+            info["spin_down_info"] = spin_down_info
+            if plot:
+                for i, j in info.items():
+                    if "spin" in i:
+                        for m, n in j.items():
+
+                            if "up" in i:
+                                plt.plot(info["energy"], n, label=m)
+                            if "down" in i:
+                                plt.plot(info["energy"], n)
+                plt.legend()
+        return info
+
+
+def parse_raman_dat(
+    vasp_raman_path="RAMANDIR-bulk@JVASP-1002_mp-149/vasp_raman.dat",
+):
+    """
+    Parse vasp_raman.dat .
+
+    generated by https://github.com/raman-sc/VASP
+    """
+    f = open(vasp_raman_path, "r")
+    lines = f.read().splitlines()
+    f.close()
+    info = {}
+    freqs = []
+    activity = []
+    alpha = []
+    beta2 = []
+    for i in lines:
+        if "#" not in i:
+            tmp = i.split()
+            freqs.append(float(tmp[1]))
+            alpha.append(float(tmp[2]))
+            beta2.append(float(tmp[3]))
+            activity.append(float(tmp[4]))
+    freqs = np.array(freqs)
+    activity = np.array(activity)
+    indices = np.arange(0, len(activity) - 1)
+    try:
+        indices = ss.find_peaks_cwt(activity, np.arange(1, 5))
+    except Exception:
+        print("Cannot use peak finding module", vasp_raman_path)
+        pass
+    info["freqs"] = freqs
+    info["activity"] = activity
+    info["alpha"] = alpha
+    info["beta2"] = beta2
+    info["indices"] = indices
+    return info
